@@ -1,5 +1,6 @@
 # -*- coding: utf-8 -*-
 from django.db import models
+from django.db.models import Q
 from django.db.models.query import QuerySet
 from django.core.exceptions import MultipleObjectsReturned, ObjectDoesNotExist
 from django.conf import settings
@@ -13,6 +14,7 @@ import logging
 
 log = logging.getLogger('vkontakte_groups_migration')
 
+
 def ModelQuerySetManager(ManagerBase=models.Manager):
     '''
     Function that return Manager for using QuerySet class inside the model definition
@@ -30,6 +32,7 @@ def ModelQuerySetManager(ManagerBase=models.Manager):
 
     return Manager()
 
+
 class GroupMigrationQueryset(object):
 
     @property
@@ -84,6 +87,7 @@ class GroupMigrationManager(models.Manager, GroupMigrationQueryset):
             yield (offset+len(ids), response['count'], offset_step)
 
         # save stat with time and other fields
+        stat.time = datetime.now()
         stat.save_final()
         signals.group_migration_updated.send(sender=GroupMigration, instance=stat)
 
@@ -149,6 +153,22 @@ class GroupMigration(models.Model):
         except IndexError:
             return None
 
+    @property
+    def user_ids(self):
+        return GroupMembership.objects.filter(group=self.group) \
+            .filter(Q(time_entered=None, time_left=None) | \
+                    Q(time_entered=None, time_left__gt=self.time) | \
+                    Q(time_entered__lte=self.time, time_left=None) | \
+                    Q(time_entered__lte=self.time, time_left__gt=self.time)).values_list('user_id', flat=True)
+
+    @property
+    def entered_user_ids(self):
+        return GroupMembership.objects.filter(group=self.group).filter(time_entered=self.time).values_list('user_id', flat=True)
+
+    @property
+    def left_user_ids(self):
+        return GroupMembership.objects.filter(group=self.group).filter(time_left=self.time).values_list('user_id', flat=True)
+
     def delete(self, *args, **kwargs):
         '''
         Recalculate next stat members instance
@@ -163,6 +183,32 @@ class GroupMigration(models.Model):
         self.hidden = True
         self.save()
 
+    def fix_memberships(self):
+
+        if self.next:
+            # delete memberships stopped in current and started from the next migration, we need to join them
+            GroupMembership.objects.filter(group=self.group, user_id__in=self.members_left_ids,
+                time_left=None, time_entered=self.next.time).delete()
+
+            # move left users to the time of the next migration
+            GroupMembership.objects.filter(group=self.group, user_id__in=self.members_left_ids, time_left=self.time) \
+                .exclude(user_id__in=self.next.members_ids) \
+                .update(time_left=self.next.time)
+
+            # these users not entered actually -> delete them
+            GroupMembership.objects.filter(group=self.group, user_id__in=self.members_entered_ids, time_entered=self.time) \
+                .exclude(user_id__in=self.next.members_ids).delete()
+
+            # update entered_time of all entered users to the time of next migration
+            GroupMembership.objects.filter(group=self.group, user_id__in=self.members_entered_ids, time_entered=self.time) \
+                .filter(user_id__in=self.next.members_ids).update(time_entered=self.next.time)
+        else:
+            # if no next migration -> delete all current entered users
+            GroupMembership.objects.filter(group=self.group, user_id__in=self.members_entered_ids, time_entered=self.time).delete()
+
+        # update rest of left users -> they are not left
+        GroupMembership.objects.filter(group=self.group, user_id__in=self.members_left_ids, time_left=self.time).update(time_left=None)
+
     def update_next(self):
         next_stat = self.next
         if next_stat:
@@ -177,13 +223,18 @@ class GroupMigration(models.Model):
         super(GroupMigration, self).save(*args, **kwargs)
 
         if update_next:
+            self.fix_memberships()
             self.update_next()
 
     def save_final(self):
-        self.time = datetime.now()
+        '''
+        Update local fields, update memberships models and save model,
+        Recommended to use in the last moment of creating migration
+        '''
         self.offset = 0
         self.clean_members()
         self.update()
+        self.update_users_memberships()
         self.save()
 
     def compare_with_previous(self):
@@ -206,12 +257,12 @@ class GroupMigration(models.Model):
         try:
             assert not self.hidden
             assert 'vkontakte_groups_statistic' in settings.INSTALLED_APPS
-            members_count = self.group.statistics.get(date=self.time.date()).members
+            members_count = self.group.statistics.get(date=self.time.date(), period=1).members
         except (ObjectDoesNotExist, AssertionError):
             return
 
         division = float(self.members_count) / members_count
-        if 0.99 < division < 1.01:
+        if 0.98 < division < 1.02:
             return
         else:
             log.warning("Suspicious migration found. Statistic value is %d, API value is %d. Group %s, migration ID %d" % (members_count, self.members_count, self.group, self.id))
@@ -224,12 +275,12 @@ class GroupMigration(models.Model):
         self.members_ids = list(set(self.members_ids))
 
     def update(self):
-        self.update_migration()
-        self.update_deactivated()
-        self.update_with_avatar()
+        self.update_left_entered()
+#        self.update_deactivated()
+#        self.update_with_avatar()
         self.update_counters()
 
-    def update_migration(self):
+    def update_left_entered(self):
         prev_stat = self.prev
         if self.prev and self.group:
             self.members_left_ids = list(set(prev_stat.members_ids).difference(set(self.members_ids)))
@@ -252,7 +303,7 @@ class GroupMigration(models.Model):
         Fetch all users of group, make new m2m relations, remove old m2m relations
         '''
         log.debug('Fetching users for the group "%s"' % self.group)
-        User.remote.fetch(ids=self.members_ids, only_expired=True)
+        User.remote.fetch(ids=self.user_ids, only_expired=True)
 
         # process entered nad left users of the group
         # here is possible using relative self.members_*_ids, but it's better absolute values, calculated by self.group.users
@@ -260,7 +311,6 @@ class GroupMigration(models.Model):
         ids_left = set(ids_current).difference(set(self.members_ids))
         ids_entered = set(self.members_ids).difference(set(ids_current))
 
-        # TODO: refactor with through table
         log.debug('Adding %d new users to the group "%s"' % (len(ids_entered), self.group))
         ids = User.objects.filter(remote_id__in=ids_entered).values_list('pk', flat=True)
         self.group.users.through.objects.bulk_create([self.group.users.through(group_id=self.group.pk, user_id=id) for id in ids])
@@ -273,4 +323,73 @@ class GroupMigration(models.Model):
         log.info('Updating m2m relations of users for group "%s" successfuly finished' % self.group)
         return True
 
+    def update_users_memberships(self):
+        '''
+        Fetch all users of group, make new m2m relations, remove old m2m relations
+        '''
+        if self.hidden:
+            return
+
+        if not self.prev:
+            # it's first migration -> create memberships
+            GroupMembership.objects.bulk_create([GroupMembership(group=self.group, user_id=user_id) for user_id in self.members_ids])
+        else:
+            # ensure current number of memberships equal to members in previous migration
+            memberships_count = GroupMembership.objects.filter(group=self.group, time_left=None).count()
+            if self.prev.members_count != memberships_count:
+                # something wrong with previous migration
+                raise Exception("Number of current memberships %d is not equal to members count %d of previous migration, group %s at %s" % (memberships_count, self.prev.members_count, self.group, self.time))
+
+            # ensure entered users not in memberships now
+            error_ids_count = GroupMembership.objects.filter(group=self.group, time_left=None, user_id__in=self.members_entered_ids).count()
+            if error_ids_count != 0:
+                raise Exception("Found %d just entered users, that still not left from the group %s at %s" % (error_ids_count, self.group, self.time))
+
+            # ensure left users in memberships now
+            left_ids_count = GroupMembership.objects.filter(group=self.group, time_left=None, user_id__in=self.members_left_ids).count()
+            if left_ids_count != self.members_left_count:
+                raise Exception("Not all left users found %d != %d between active in group %s at %s" % (left_ids_count, self.members_left_count, self.group, self.time))
+
+            # create entered users
+            GroupMembership.objects.bulk_create([GroupMembership(group=self.group, user_id=user_id, time_entered=self.time) for user_id in self.members_entered_ids])
+
+            # update left users
+            GroupMembership.objects.filter(group=self.group, time_left=None, user_id__in=self.members_left_ids).update(time_left=self.time)
+
+        return True
+
+class GroupMembershipManager(models.Manager):
+
+    def get_user_ids_of_period(self, group, date_from, date_to, field=None):
+
+        if field is None:
+            # TODO: made normal filtering
+            kwargs = {'time_entered': None, 'time_left': None} \
+                | {'time_entered__lte': date_from, 'time_left': None} \
+                | {'time_entered': None, 'time_left__gte': date_to}
+        elif field in ['left','entered']:
+            kwargs = {'time_%s__gt' % field: date_from, 'time_%s__lte' % field: date_to}
+        else:
+            raise ValueError("Attribute `field` should be equal to 'left' of 'entered'")
+
+        return self.filter(group=group, **kwargs).order_by('user_id').distinct('user_id').values_list('user_id', flat=True)
+
+    def get_active_users_ids_of_date(self, group, date):
+        return self.filter(group=group, **kwargs).order_by('user_id').distinct('user_id').values_list('user_id', flat=True)
+
+class GroupMembership(models.Model):
+    class Meta:
+        verbose_name = u'Членство пользователя группы Вконтакте'
+        verbose_name_plural = u'Членства пользователей групп Вконтакте'
+        unique_together = (('group','user_id','time_entered'), ('group','user_id','time_left'),)
+        ordering = ('group', 'user_id', 'id')
+
+    group = models.ForeignKey(Group, verbose_name=u'Группа', related_name='memberships')
+    user_id = models.PositiveIntegerField(u'ID пользователя', db_index=True)
+
+    time_entered = models.DateTimeField(u'Дата и время вступления', null=True, db_index=True)
+    time_left = models.DateTimeField(u'Дата и время выхода', null=True, db_index=True)
+
+    objects = GroupMembershipManager()
+
 import signals
